#ifndef IMAGE_INCLUDED
#define IMAGE_INCLUDED

#define SUCCESS 0
#define INVALID_INPUT_FORMAT -1
#define IMAGE_CORRUPTED -2
#define OUT_OF_MEMORY -3
#define FILE_NOT_FOUND -4
#define NOT_YET_IMPLEMENTED_FEATURES_DETECTED -5
#define GZIP_ERROR -6
#define INTERNAL_ERROR -7

#define WHITE 18
#define BLACK 19

struct CompressedImage {
	unsigned int size; //in bytes
	unsigned char* image; //compressed image data
};

struct Bitmap {
	unsigned int size[2]; //width;height
	unsigned int* image; //left2right;top2bottom;RGB
};

struct PietImage {
	unsigned int size[2]; //width;height (CODEL SIZE IS 1)
	unsigned char* image; //left2right;top2bottom;range of values:0-20 (20: invalid color @ input image)
};

extern const unsigned int PIET_COLORS[]; // contains RGB-values of piet-image

int readFile(const char* filename, CompressedImage& output);

inline void GIF_writeData(unsigned int pixelValue, unsigned int readPos, unsigned int i_X, unsigned int i_Y, unsigned int i_Width, unsigned int i_Height, unsigned int* imageData, unsigned int width, bool interlaced); //TODO: include in gif2bitmap-function
bool PPM_readAsci(unsigned int& value, CompressedImage input, unsigned int& position);

int gif2bitmap(CompressedImage input, Bitmap& output);
int png2bitmap(CompressedImage input, Bitmap& output);
int bmp2bitmap(CompressedImage input, Bitmap& output);
int ppm2bitmap(CompressedImage input, Bitmap& output);

int compressed2bitmap(CompressedImage input, Bitmap& output);

unsigned int getCodelSize(Bitmap input, bool unknownPixelsAsWhite=false);
bool isValidCodelSize(unsigned int codelsize, Bitmap input, bool unknownPixelsAsWhite=false);
unsigned char getPietColor(unsigned int rgb, bool unknownPixelsAsWhite=false);

int bitmap2pietimage(Bitmap input, PietImage& output, unsigned int codelSize=0, bool unknownPixelsAsWhite=false);

#endif

