#ifndef PROGRAM_INCLUDED
#define PROGRAM_INCLUDED

#define NOP 0
#define PUSH 1
#define POP 2
#define ADD 3
#define SUB 4
#define MUL 5
#define DIV 6
#define MOD 7
#define NOT 8
#define GRT 9
#define PNTR 10
#define SWTCH 11
#define DUP 12
#define ROLL 13
#define INN 14
#define INC 15
#define OUTN 16
#define OUTC 17
#define EXIT 18

struct ProgramCommandList {
	ProgramCommandList* next; // order in assembly file, brainfuck file, ... USE IT TO REACH EVERY COMMAND AT LEAST ONCE
	unsigned char command; // PUSH; POP; ADD; ...
	unsigned int parameter; // only for "PUSH" command
	unsigned int label; // normally 0; only !=0 if this command is reachable via a JUMP by other commands
	unsigned int id; //ID in list. Starts with 0. If you go through this list using ->next, id should increase by 1 for every step you do.
	/**
	 * Stores the next commands in the program flow.
	 * Index 0: Next command in normal program flow. only NULL if program will terminate after the current command.
	 * Index 1: Next command for "pointer 1" OR "switch 1". Otherwise 0.
	 * Index 2: Next command for "pointer 2". Otherwise 0.
	 * Index 3: Next command for "pointer 3". Otherwise 0.
	 */
	ProgramCommandList* cond[4]; //
};


struct Program{
	ProgramCommandList* commands;
	unsigned int maxLabel; //maximum label used.
	unsigned int numCmds; //commands in list. equal to maximum ID +1.
};

#endif

