#include "config.h"

#include "Image.h"
#include "PietImage2Program.h"
#include "Interpreter.h"

#include <stdio.h>
#include <stdlib.h>


char* inputFileName;
bool unknownColorsAsBlack;
int codelSize;


bool cmp(const char* s1, const char* s2){
	return cmp((const unsigned char*)s1, (const unsigned char*)s2);
}

bool cmp(const unsigned char* s1, const unsigned char* s2){
	if (s1 == s2)
		return true;
	if (s1 == 0 || s2 == 0)
		return false;
	while(*s1 != 0 && *s2 != 0){
		if (*(s1++) != *(s2++))
			return false;
	}
	if (*s1 != 0 || *s2 != 0)
		return false;
	return true;
}

bool parseArgs(int argc, char* argv[]){
	codelSize = 0;
	unknownColorsAsBlack = false;
	inputFileName = 0;
	bool codelSizeSet = false;
	while (--argc > 0){
		argv++;
		if (cmp(*argv,"-b") || cmp(*argv,"-ub")){
			unknownColorsAsBlack = true;
			continue;
		}
		if (cmp(*argv,"-c") || cmp(*argv,"-cs")){
			if (!codelSizeSet){
				codelSizeSet = true;
				if (--argc > 0){
					argv++;
					codelSize = atoi(*argv);
				}else
					return false;
			}else
				return false;
			continue;
		}
		//add more parameters here...
		if (inputFileName == 0){
			inputFileName = *argv;
		}else
			return false;
	}
	return (inputFileName != 0);
}

void showUsage(const char* progName){
	printf("fpiet v%d.%d%s by Matthias Ernst\n\n",MAJOR_VERSION,MINOR_VERSION,SPECIAL_VERSION);
	printf("Usage: %s [options] file\n",progName);
	printf("Options:\n");
	printf("\t-b            - Unknown colors are black\n");
	printf("\t-c <n>        - Codelsize (0: auto)\n");
}

int main(int argc, char* argv[]){

	if (!parseArgs(argc, argv)){
		showUsage(*argv);
		return -1;
	}

	//TODO: BMP with colormap, RLE
	//TODO: Binary PPM

	CompressedImage compressedimage;
	compressedimage.image = 0;
	compressedimage.size = 0;
	if (SUCCESS!=readFile(inputFileName, compressedimage)){
		printf("Cannot read file: %s\n",inputFileName);
		return -1;
	}
	Bitmap bmp;
	bmp.image = 0;
	bmp.size[0] = 0;
	bmp.size[1] = 0;
	if (SUCCESS!=compressed2bitmap(compressedimage, bmp)){
		printf("Unsupported file format or corrupted input file.\n");
		return -1;
	}
	delete compressedimage.image;
	PietImage pietimage;
	pietimage.image = 0;
	pietimage.size[0] = 0;
	pietimage.size[1] = 0;
	if (SUCCESS!=bitmap2pietimage(bmp, pietimage, ::codelSize, !::unknownColorsAsBlack)){
		printf("Not enough memory or other internal error.\n");
		return -1;
	}
	delete bmp.image;
	Program program;
	program.commands = 0;
	program.numCmds = 0;
	program.maxLabel = 0;
	if (SUCCESS!=pietimage2program(pietimage, program)){
		printf("Not enough memory or other internal error.\n");
		return -1;
	}
	delete pietimage.image;

	runProgram(program);
	return 1;
}
